from flask_shop import db
from werkzeug.security import generate_password_hash, check_password_hash
from datetime import datetime


class BaseModel:
    # 创建时间
    create_time = db.Column(db.DateTime, default=datetime.now)
    # 更新时间
    update_time = db.Column(
        db.DateTime, default=datetime.now, onupdate=datetime.now)


class User(db.Model, BaseModel):
    __tablename__ = "t_user"
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(32), unique=True, nullable=False)
    pwd = db.Column(db.String(128))
    nick_name = db.Column(db.String(32))
    phone = db.Column(db.String(11))
    email = db.Column(db.String(32))

    role_id = db.Column(db.Integer, db.ForeignKey('t_role.id'))

    # getter

    @property
    def password(self):
        print(self.pwd)
        return pwd

    @password.setter
    def password(self, t_pwd):
        # 加密
        self.pwd = generate_password_hash(t_pwd)

    # 进行密码比对
    def check_password(self, t_pwd):
        return check_password_hash(self.pwd, t_pwd)

    # 响应给前端字典(json数据格式)
    def to_dict(self):
        return {
            "id": self.id,
            "name": self.name,
            "nick_name": self.nick_name,
            "phone": self.phone,
            "email": self.email,
            "role_name": self.role.name if self.role else ''
        }


# 创建一个中间表，形成角色和权限之间多对多的关系
trm = db.Table('t_role_menu',
               db.Column('rid', db.Integer, db.ForeignKey('t_role.id')),
               db.Column('mid', db.Integer, db.ForeignKey('t_menu.id'))
               )

# 菜单model的创建


class Menu(db.Model):
    __tablename__ = 't_menu'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(32), unique=True, nullable=False)
    level = db.Column(db.Integer)
    path = db.Column(db.String(32))

    pid = db.Column(db.Integer, db.ForeignKey('t_menu.id'))
    children = db.relationship('Menu')

    roles = db.relationship('Role', secondary=trm)

    # 响应给前端字典(json数据格式)
    def to_dict(self):
        return {
            "id": self.id,
            "name": self.name,
            "level": self.level,
            "path": self.path,
            "pid": self.pid
            # "children": self.get_child_list(),
        }

    # 获取菜单里面的子菜单
    def get_child_list(self):
        data = []
        # 获取所有的子菜单
        obj_child = self.children
        # 遍历获取每个子菜单的对象
        for o in obj_child:
            data.append(o.to_dict())
        # 返回所有子菜单对象
        # print(data)
        return data


# 角色管理的实现
class Role(db.Model):
    __tablename__ = "t_role"
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(32), unique=True, nullable=False)
    desc = db.Column(db.String(32))

    users = db.relationship('User', backref='role')
    menus = db.relationship('Menu', secondary=trm)

    def to_dict(self):
        return {
            'id': self.id,
            'name': self.name,
            'desc': self.desc,
            'menu': self.get_menu_dict()    # 返回所有菜单的子级
        }

    def get_menu_dict(self):
        menu_list = []
        """
        排序：
            前端页面显示的权限信息是无序的，
            所以要在后端将信息进行排序。
            sorted方法用来进行排序操作，如数字，字符串组成的list，根据dict类型的key或者value排序。
            self.menus 是一个对象列表
            sorted(self.menus, key=lambda temp: temp.id)
        """
        menus = sorted(self.menus, key=lambda temp: temp.id)
        for m in menus:
            if m.level == 1:
                # 第一级菜单
                first_dict = m.to_dict()
                # 为二级子菜单创建的列表
                first_dict['children'] = []
                for s in menus:
                    if s.level == 2 and s.pid == m.id:
                        first_dict['children'].append(s.to_dict())
                menu_list.append(first_dict)
        return menu_list


# 商品分类模型
class Category(db.Model):
    __tablename__ = 't_category'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(32), nullable=False)
    level = db.Column(db.Integer)

    pid = db.Column(db.Integer, db.ForeignKey('t_category.id'))
    children = db.relationship('Category')

    # 与attribute的关联关系
    attributes = db.relationship('Attribute', backref='category')

    # 响应给前端字典(json数据格式)

    def to_dict(self):
        return {
            "id": self.id,
            "name": self.name,
            "level": self.level,
            "pid": self.pid
        }


# 商品属性模型
class Attribute(db.Model):
    __tablename__ = 't_attribute'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(32))
    val = db.Column(db.String(255))
    # 枚举类型
    _type = db.Column(db.Enum('static', 'dynamic'))

    # 外键，关联分类
    cid = db.Column(db.Integer, db.ForeignKey('t_category.id'))

    # 响应给前端字典(json数据格式)
    def to_dict(self):
        return {
            "id": self.id,
            "name": self.name,
            "val": self.val,
            "type": self._type,
            "cid": self.cid
        }


# 商品模型
class Goods(db.Model):
    __tablename__ = 't_goods'
    id = db.Column(db.Integer, primary_key=True)
    # 商品名字
    name = db.Column(db.String(512))
    # 价格
    price = db.Column(db.Float)
    # 库存
    number = db.Column(db.Integer)
    # 商品介绍
    introduce = db.Column(db.Text)

    # 图片保存的是图片地址
    big_log = db.Column(db.String(256))
    small_log = db.Column(db.String(256))
    # 状态
    state = db.Column(db.Integer)   # 0通过 1审核中 2已审核
    # 是否促销
    is_promte = db.Column(db.Integer)
    # 促销个数
    hot_number = db.Column(db.Integer)
    # 商品排名
    weight = db.Column(db.Integer)
    # 外键 商品分类
    cid_one = db.Column(db.Integer, db.ForeignKey('t_category.id'))
    cid_two = db.Column(db.Integer, db.ForeignKey('t_category.id'))
    cid_three = db.Column(db.Integer, db.ForeignKey('t_category.id'))

    # 通过商品查找分类 关联关系
    category = db.relationship('Category', foreign_keys=[cid_three])

    def to_dict(self):
        return {
            'id': self.id,
            'name': self.name,
            'price': self.price,
            'number': self.number,
            'introduce': self.introduce,
            'big_log': self.big_log,
            'small_log': self.small_log,
            'state': self.state,
            'is_promote': self.is_promte,
            'hot_number': self.hot_number,
            'weight': self.weight,
            'cid_one': self.cid_one,
            'cid_two': self.cid_two,
            'cid_three': self.cid_three,
            'attrs': [a.to_dict() for a in self.category.attributes]
        }


# 商品图片模型
class Picture(db.Model):
    __tablename__ = 't_picture'
    id = db.Column(db.Integer, primary_key=True)
    # 图片存放路径
    path = db.Column(db.String(512))
    # 图片是哪个商品的图片
    gid = db.Column(db.Integer, db.ForeignKey('t_goods.id'))

    # 关联关系
    goods = db.relationship('Goods', backref='pictures')


# 商品属性模型，中间表
class GoodsAttr(db.Model):
    __tablename__ = 't_goods_attr'
    # 是哪个商品的属性
    gid = db.Column(db.Integer, db.ForeignKey('t_goods.id'), primary_key=True)
    # 这个属性属于哪个商品
    aid = db.Column(db.Integer, db.ForeignKey(
        't_attribute.id'), primary_key=True)

    """
    因为新添加的商品属性跟原来的商品属性是不一样的，所以从新增加一个属性字段
    """
    val = db.Column(db.String(512))
    # 什么类型的参数
    _type = db.Column(db.String(8))
