"""
User视图函数
"""

from flask_shop.user import user_bp, user_api

from flask_shop import models
from flask import request
from flask_shop import db
from flask_restful import Resource, reqparse
from flask_shop.utils.message import state_message
from flask_shop.utils.tokens import generate_auth_token, verify_auth_token
from flask_shop.utils.tokens import login_required
import re

@user_bp.route('/')
def index():
    return "User Hello"


# 用户注册操作
class User(Resource):
    def get(self):
        try:
            id = int(request.args.get('id').strip())
            usr = models.User.query.filter_by(id=id).first()
            if usr:
                return state_message(status=200, data=usr.to_dict(), message="获取信息成功！")
            else:
                return state_message(status=200, data=[], message="没有此用户！")
        except Exception as e:
            print(e)
            return state_message(10000)
    # 添加用户信息
    def post(self):
        
        name = request.form.get('name')
        pwd = request.form.get('pwd')
        confirm_pwd = request.form.get('confirm_pwd')
        nick_name = request.form.get('nick_name')
        phone = request.form.get('phone')
        email = request.form.get('email')
        role_id = int(request.form.get('role_name')) if request.form.get('role_name') else 0

        # 验证数据的正确性
        if not all([name, pwd, confirm_pwd]):
            return state_message(10000)
        if len(name) < 2:
            return state_message(10011)
        if len(pwd) < 2:
            return state_message(10012)
        if pwd != confirm_pwd:
            return state_message(10013)
        if not re.match(r"^(13[0-9]|14[5|7]|15[0|1|2|3|5|6|7|8|9]|18[0|1|2|3|5|6|7|8|9])\d{8}$", phone):
            return state_message(10014)
        if not re.match(r"^\w+([-+.]\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*$", email):
            return state_message(10015)
        # 添加数据
        try:
            usr = models.User(name=name, password=pwd, nick_name=nick_name, phone=phone, email=email, role_id=role_id)
            db.session.add(usr)
            db.session.commit()
            return state_message(200)
        except Exception:
            return state_message(20000)

    # 修改用户信息
    def put(self):
        try:
            id = int(request.form.get('id').strip())
            email = request.form.get('email').strip() if request.form.get('email') else ''
            phone = request.form.get('phone').strip() if request.form.get('phone') else ''
            role_id = int(request.form.get('role_name')) if request.form.get('role_name') else 0
            usr = models.User.query.get(id)
            if usr:
                usr.email = email
                usr.phone = phone
                usr.role_id = role_id
                db.session.commit()
                return state_message(200, message="修改用户信息成功！")
            else:
                return state_message(10018)
        except Exception as e:
            print(e)
            return state_message(10000)
    
    # 删除用户信息
    def delete(self):
        try:
            id = request.json.get('id')
            usr = models.User.query.get(id)
            if usr:
                db.session.delete(usr)
                db.session.commit()
                return state_message(200, message="删除信息成功！")
            else:
                return state_message(10019)
        except Exception as e:
            print(e)
            return state_message(20000)

# 添加类视图和url
user_api.add_resource(User, '/user') 
            


class UserList(Resource):
    def get(self):
        # 创建解析对象
        parser = reqparse.RequestParser()
        # 利用解析对象添加需要验证的参数
        parser.add_argument('name', type=str)
        # 当前页码
        parser.add_argument('pnum', type=int)
        # 每页显示条数
        parser.add_argument('psize', type=int)
        # 3.利用解析器对象进行验证，若正确，直接返回验证后合格的参数值，若错误，抛异常信息给客户端
        try:
            args = parser.parse_args()
            name = args.get('name')
            pnum = args.get('pnum') if args.get('pnum') else 1
            psize = args.get('psize') if args.get('psize') else 2
            # 如果name有值
            if name:
                users_p = models.User.query.filter(models.User.name.like(f'%{name}%')).paginate(pnum, psize)
            else:
                users_p = models.User.query.paginate(pnum, psize)
            data = {
                'pnum': pnum,
                'totalPage': users_p.total,
                'users': [u.to_dict() for u in users_p.items]
            }
            return state_message(200, data=data, message='获取用户列表成功！')
        except Exception as e:
            print(e)
            return state_message(1000)

user_api.add_resource(UserList, '/user_list')

# 用户端登录操作
@user_bp.route('/login', methods=['POST'])
def login():
    name = request.form.get('name')
    pwd = request.form.get('pwd')

    if not all([name, pwd]):    # 一个为False就都为False
        """如果非True"""
        return state_message(10000)
    if len(name) > 1:
        # 获取数据库的信息
        # usr = db.session.query(models.User).filter_by(name=name).first()
        # usr 是 User对象
        usr = models.User.query.filter_by(name=name).first()
        print(usr)
        if usr:
            if usr.check_password(pwd):     # 返回True或者False
                # 生成token
                token = generate_auth_token(usr.id, 10000000)
                # 解密token
                verify_auth_token(token)
                return state_message(200, data={'token': token})

    return {'status':10001,'msg':'用户名或密码错误'}


# 重置密码
@user_bp.route('/reset', methods=['GET'])
def reset():
    try:
        id = int(request.args.get('id'))
        usr = models.User.query.get(id)
        usr.password = '123'
        db.session.commit()
        return state_message(200, message="重置密码成功！")
    except Exception as e:
        return state_message(20000)



@user_bp.route('/test')
@login_required     # test = login_required(test)
def test():
    return state_message(200)
    